@@ -6,7 +6,7 @@ use warnings;
 use Class::MOP;
 use Scalar::Util 'blessed', 'looks_like_number';
 
-our $VERSION   = '0.93';
+our $VERSION   = '0.94';
 $VERSION = eval $VERSION;
 our $AUTHORITY = 'cpan:STEVAN';
 
@@ -14,7 +14,12 @@ sub Value { defined($_[0]) && !ref($_[0]) }
 
 sub Ref { ref($_[0]) }
 
-sub Str { defined($_[0]) && ref(\$_[0]) eq 'SCALAR' }
+# We need to use a temporary here to flatten LVALUEs, for instance as in
+# Str(substr($_,0,255)).
+sub Str {
+    my $value = $_[0];
+    defined($value) && ref(\$value) eq 'SCALAR'
+}
 
 sub Num { !ref($_[0]) && looks_like_number($_[0]) }
 
@@ -103,9 +108,7 @@ no user serviceable parts inside.
 
 =head1 BUGS
 
-All complex software has bugs lurking in it, and this module is no
-exception. If you find a bug please either email me, or add the bug
-to cpan-RT.
+See L<Moose/BUGS> for details on reporting bugs.
 
 =head1 AUTHOR
 
